==> shadowwalker-1.3.0-py3-none-any.whl: shadowwalker/v2/core.py <==
from omnitools import def_template, IS_WIN32, abs_dir
from subprocess import Popen, PIPE
from .utils import ping
import threadwrapper
import threading
import requests
import datetime
import shutil
import random
import yaml
import stat
import time
import json
import os


pkg_data_dir = os.path.join(abs_dir(__file__), "pkg_data")
# content has to be clash proxies yaml format
SOURCES = [
    "https://proxies.bihai.cf/clash/proxies",
    "https://sspool.herokuapp.com/clash/proxies",
    "https://hellopool.herokuapp.com/clash/proxies",
    "https://fq.lonxin.net/clash/proxies",
    "http://guobang.herokuapp.com/clash/proxies",
    "https://free.dswang.ga/clash/proxies",
    "https://origamiboy.herokuapp.com/clash/proxies",
    "https://free886.herokuapp.com/clash/proxies",
    "https://us-proxypool.herokuapp.com/clash/proxies",
    "https://eu-proxypool.herokuapp.com/clash/proxies",
    "http://39.106.12.141:8081/clash/proxies",
    "http://wxshi.top:9090/clash/proxies",
    "https://cdn.jsdelivr.net/gh/yuewei5215/Proxies-1/sub/sub_merge_yaml.yml",
]
# clash config in yaml format to be harvested
yyyy = datetime.datetime.now().year
mm = datetime.datetime.now().month
dd = datetime.datetime.now().day
CLASHES = [
    "https://cdn.jsdelivr.net/gh/yu-steven/openit/Clash.yaml",
    "https://cdn.jsdelivr.net/gh/moreyu/clash.yaml/Clash.yaml",
    "https://cdn.jsdelivr.net/gh/Mengzhiyaa/ClashID/local.yaml",
    "https://cdn.jsdelivr.net/gh/tbbatbb/Proxy/dist/clash.config.yaml",
    "https://cdn.jsdelivr.net/gh/anaer/Sub/clash.yaml",
    "https://cdn.jsdelivr.net/gh/aiboboxx/clashfree/clash.yml",
    "https://cdn.jsdelivr.net/gh/sh3d0ww02f/sh3d0ww02f.github.io/clash1.yaml",
    "https://cdn.jsdelivr.net/gh/mohsenhafezquran/ClashVPN/Clash",
    "https://cdn.jsdelivr.net/gh/zjr13808836946/zjr_clash/v2_ssr_class",
    "https://cdn.jsdelivr.net/gh/openrunner/clash-freenode/clash.yaml",
    "https://cdn.jsdelivr.net/gh/oslook/clash-freenode/clash.yaml",
    "https://nodefree.org/dy/{yyyy}{mm:0>2}/{yyyy}{mm:0>2}{dd:0>2}.yaml".format(yyyy=yyyy, mm=mm, dd=dd-1),
    "https://cdn.jsdelivr.net/gh/pojiezhiyuanjun/freev2/{:0>2}{:0>2}clash.yml".format(mm, dd-2),
]


class ShadowWalker:
    prev_proxy = None
    clash = None
    clash_port = 7890
    terminate = False
    blacklist = [
        # "38.114.114.",
    ]
    multiple = False
    is_bench_mega = False
    is_bench_pixeldrain = False

    @property
    def proxy(self):
        return "127.0.0.1:{}".format(self.clash_port)

    def __init__(
            self,
            clash_bin=os.path.join(pkg_data_dir, "clash.{}".format("exe" if IS_WIN32 else "bin")),
            exclude_country=["CN"],
            proxies=None,
            max_ping=171,
            min_step=0.75
    ):
        self.clash_bin = clash_bin
        if not IS_WIN32:
            t = os.stat(self.clash_bin)
            os.chmod(self.clash_bin, t.st_mode | stat.S_IEXEC)
        self.exclude_country = exclude_country
        self.proxies = proxies
        self.max_ping = max_ping
        self.min_step = min_step
        self.lock = threading.Lock()
        if not self.proxies:
            self.update_proxies(max_ping=max_ping, min_step=min_step)

    def clone(self):
        self.multiple = True
        sw = ShadowWalker(
            proxies=self.proxies
        )
        sw.multiple = True
        sw.clash = None
        sw.clash_bin = self.clash_bin
        sw.exclude_country = self.exclude_country
        sw.clash_port = self.clash_port+10
        sw.max_ping = self.max_ping
        return sw

    def update_proxies(self, max_ping, min_step):
        self.proxies = []
        if not (os.path.isfile("proxies_cache.json") and os.path.getmtime("proxies_cache.json")+60*60*24 > time.time()):
            for url in SOURCES:
                for i in range(0, 10):
                    try:
                        print("\r", url, "fetching proxies, try {}/10".format(i+1), end="", flush=True)
                        r = requests.get(url, timeout=3)
                        if r.status_code == 200:
                            c = r.content
                            self.proxies.extend(yaml.safe_load(c.decode())["proxies"])
                            break
                    except:
                        pass
            for url in CLASHES:
                for i in range(0, 10):
                    try:
                        print("\r", url, "fetching clashes, try {}/10".format(i+1), end="", flush=True)
                        r = requests.get(url, timeout=3)
                        if r.status_code == 200:
                            c = r.content
                            self.proxies.extend(yaml.safe_load(c.decode())["proxies"])
                            break
                    except:
                        pass
            open("proxies_cache.json", "wb").write(json.dumps(self.proxies).encode())
        else:
            self.proxies = json.loads(open("proxies_cache.json", "rb").read().decode())
        if not self.proxies:
            raise Exception("empty proxies")
        print("\r", end="", flush=True)
        self.proxies = [_ for _ in self.proxies if not any(_["server"].startswith(ip) for ip in self.blacklist)]
        self.test_latency(self.exclude_country, max_ping=max_ping, min_step=min_step)

    def start(self, quiet: bool = False, proxy: dict = None):
        if not self.proxies and not proxy:
            raise ValueError("empty proxies")
        if not proxy:
            if len(self.fast_proxy) > 1:
                proxies = self.fast_proxy
            else:
                proxies = self.proxies
            proxy = random.SystemRandom().choice(proxies)
            proxy = random.SystemRandom().choice(proxies)
        self.prev_proxy = proxy
        id = "{}_{}".format(proxy["server"], proxy["port"])
        config_fp = os.path.join("config", "{}_{}".format(self.clash_port, id), "config_ss.yaml")
        os.makedirs(os.path.dirname(config_fp), exist_ok=True)
        data = ["cache.db", "Country.mmdb"]
        for _ in data:
            shutil.copyfile(os.path.join(pkg_data_dir, _), os.path.join(os.path.dirname(config_fp), _))
        _proxy = proxy.copy()
        _proxy["name"] = "PROXY"
        open(config_fp, "wb").write(('''\
port: {}
allow-lan: true
mode: rule
proxies:
  - {}
rules:
  - MATCH,PROXY
'''.format(
            self.clash_port,
            "\n".join("{}{}".format("    " if i else "", _) for i, _ in enumerate(yaml.dump(_proxy).splitlines())),
        )).encode())
        if not quiet:
            self.clash = Popen([self.clash_bin, "-f", config_fp, "-d", os.path.dirname(config_fp)], close_fds=not IS_WIN32)
        else:
            self.clash = Popen([self.clash_bin, "-f", config_fp, "-d", os.path.dirname(config_fp)], stdout=PIPE, stderr=PIPE, close_fds=not IS_WIN32)
        pid = self.clash.pid
        if self.multiple:
            print("\rstarted clash", self.clash_port, id, "pid", pid, end="")
        else:
            print("started clash", self.clash_port, id, "pid", pid)
        while not self.terminate:
            time.sleep(1)
        self.clash.terminate()
        time.sleep(1)
        shutil.rmtree(os.path.dirname(config_fp))
        self.terminate = False
        self.clash = None

    def start_fast_proxy(self, *args, **kwargs):
        return self.start(*args, **kwargs)

    def start_gse_proxy(self, quiet: bool = False):
        if not self.gse_proxy:
            raise ValueError("empty proxies")
        proxies = self.gse_proxy
        proxy = random.SystemRandom().choice(proxies)
        proxy = random.SystemRandom().choice(proxies)
        return self.start(quiet=quiet, proxy=proxy)

    def stop(self):
        if self.clash:
            if self.multiple:
                print("\rclash terminating", self.clash.pid, end="")
            else:
                print("clash terminating", self.clash.pid)
            self.terminate = True
            while self.terminate:
                time.sleep(1)
            if self.multiple:
                print("\rclash terminated", end="")
            else:
                print("clash terminated")

    def test_latency(self, exclude_country, max_ping, min_step):
        print("clash testing latency, please wait")
        tw = threadwrapper.ThreadWrapper(threading.Semaphore(2**4))
        self.proxies = [_ for _ in self.proxies if "country" in _ and _["country"][-2:] not in exclude_country]
        ping_cache = {}
        if os.path.isfile("ping_cache.json") and os.path.getmtime("ping_cache.json")+60*60*24 > time.time():
            ping_cache = json.loads(open("ping_cache.json", "rb").read().decode())
        for i, proxy in enumerate(self.proxies):
            def job(i, proxy):
                try:
                    print("\r", i+1, len(self.proxies), "pinging", proxy["server"], end="", flush=True)
                    if proxy["server"] in ping_cache:
                        proxy["ping"] = ping_cache[proxy["server"]]
                    else:
                        pings = [999]
                        for i in range(0, 5):
                            p = ping(proxy["server"])
                            pings.append(p)
                            if p < 999:
                                break
                        proxy["ping"] = min(pings)
                        ping_cache[proxy["server"]] = proxy["ping"]
                    # print("\r", i+1, len(self.proxies), "pinging", proxy["server"], proxy["ping"], end="", flush=True)
                except:
                    pass
            tw.add(job=def_template(job, i, proxy))
        tw.wait()
        open("ping_cache.json", "wb").write(json.dumps(ping_cache).encode())
        def fsp(_):
            return _["server"]
            return "{}:{}".format(_["server"], _["port"])
        exp = []
        self.proxies = [_ for _ in self.proxies if _["ping"] <= max_ping]
        self.proxies = [[_,exp.append(fsp(_))][0] for _ in self.proxies if _["ping"] <= max_ping and fsp(_) not in exp]
        self.proxies = sorted(self.proxies, key=lambda x: x["ping"])
        # print(len(self.proxies))

==> shadowwalker-1.5.0-py3-none-any.whl: shadowwalker/v2/core.py <==
from omnitools import def_template, IS_WIN32, abs_dir
from cloudscraper import create_scraper
from subprocess import Popen, PIPE
from .utils import ping
import threadwrapper
import threading
import requests
import datetime
import shutil
import random
import yaml
import stat
import time
import json
import os


pkg_data_dir = os.path.join(abs_dir(__file__), "..", "pkg_data")
# content has to be clash proxies yaml format
SOURCES = [
    "https://proxies.bihai.cf/clash/proxies",
    "https://sspool.herokuapp.com/clash/proxies",
    # "https://hellopool.herokuapp.com/clash/proxies",
    # "https://fq.lonxin.net/clash/proxies",
    # "http://guobang.herokuapp.com/clash/proxies",
    # "https://free.dswang.ga/clash/proxies",
    # "https://origamiboy.herokuapp.com/clash/proxies",
    "https://free886.herokuapp.com/clash/proxies",
    "https://us-proxypool.herokuapp.com/clash/proxies",
    "https://eu-proxypool.herokuapp.com/clash/proxies",
    "http://39.106.12.141:8081/clash/proxies",
    "http://wxshi.top:9090/clash/proxies",
    "https://raw.githubusercontent.com/yuewei5215/Proxies-1/master/sub/sub_merge_yaml.yml",
]
# clash config in yaml format to be harvested
yyyy = datetime.datetime.now().year
mm = datetime.datetime.now().month
dd = datetime.datetime.now().day
CLASHES = [
    "https://cdn.jsdelivr.net/gh/yu-steven/openit/Clash.yaml",
    "https://cdn.jsdelivr.net/gh/moreyu/clash.yaml/Clash.yaml",
    "https://cdn.jsdelivr.net/gh/Mengzhiyaa/ClashID/local.yaml",
    "https://cdn.jsdelivr.net/gh/tbbatbb/Proxy/dist/clash.config.yaml",
    "https://cdn.jsdelivr.net/gh/anaer/Sub/clash.yaml",
    "https://cdn.jsdelivr.net/gh/aiboboxx/clashfree/clash.yml",
    "https://sh3d0ww02f.github.io/clash1.yaml",
    "https://cdn.jsdelivr.net/gh/mohsenhafezquran/ClashVPN/Clash",
    "https://cdn.jsdelivr.net/gh/zjr13808836946/zjr_clash/v2_ssr_class",
    "https://cdn.jsdelivr.net/gh/openrunner/clash-freenode/clash.yaml",
    "https://cdn.jsdelivr.net/gh/oslook/clash-freenode/clash.yaml",
    "https://nodefree.org/dy/{yyyy}{mm:0>2}/{yyyy}{mm:0>2}{dd:0>2}.yaml".format(yyyy=yyyy, mm=mm-1 if dd-1<1 else mm, dd=dd-1),
    "https://cdn.jsdelivr.net/gh/pojiezhiyuanjun/freev2/{:0>2}{:0>2}clash.yml".format(mm-1 if dd-2<1 else mm, dd-2),
]


class ShadowWalker:
    prev_proxy = None
    clash = None
    clash_port = 7890
    terminate = False
    blacklist = [
        # "38.114.114.",
    ]
    multiple = False
    is_bench_mega = False
    is_bench_pixeldrain = False

    @property
    def proxy(self):
        return "127.0.0.1:{}".format(self.clash_port)

    def __init__(
            self,
            clash_bin=os.path.join(pkg_data_dir, "clash.{}".format("exe" if IS_WIN32 else "bin")),
            exclude_country=["CN"],
            proxies=None,
            max_ping=171
    ):
        self.clash_bin = clash_bin
        if not IS_WIN32:
            t = os.stat(self.clash_bin)
            os.chmod(self.clash_bin, t.st_mode | stat.S_IEXEC)
        self.exclude_country = exclude_country
        self.proxies = proxies
        self.max_ping = max_ping
        self.lock = threading.Lock()
        if not self.proxies:
            self.update_proxies(max_ping=max_ping)

    def clone(self):
        self.multiple = True
        sw = ShadowWalker(
            proxies=self.proxies
        )
        sw.multiple = True
        sw.clash = None
        sw.clash_bin = self.clash_bin
        sw.exclude_country = self.exclude_country
        sw.clash_port = self.clash_port+10
        sw.max_ping = self.max_ping
        return sw

    def update_proxies(self, max_ping):
        self.proxies = []
        if not (os.path.isfile("proxies_cache.json") and os.path.getmtime("proxies_cache.json")+60*60*24 > time.time()):
            for url in SOURCES:
                for i in range(0, 5):
                    try:
                        print("\r", url, "fetching proxies, try {}/5".format(i+1), end="", flush=True)
                        r = create_scraper().get(url, timeout=3)
                        if r.status_code == 200:
                            c = r.content
                            self.proxies.extend(yaml.safe_load(c.decode())["proxies"])
                            break
                    except:
                        pass
                print()
            for url in CLASHES:
                for i in range(0, 5):
                    try:
                        print("\r", url, "fetching clashes, try {}/5".format(i+1), end="", flush=True)
                        r = create_scraper().get(url, timeout=3)
                        if r.status_code == 200:
                            c = r.content.decode()
                            c = "\n".join(_ for _ in c.splitlines() if "!<str>" not in _)
                            self.proxies.extend(yaml.safe_load(c)["proxies"])
                            break
                    except:
                        import traceback
                        traceback.print_exc()
                        break
                print()
            open("proxies_cache.json", "wb").write(json.dumps(self.proxies).encode())
        else:
            self.proxies = json.loads(open("proxies_cache.json", "rb").read().decode())
        if not self.proxies:
            raise Exception("empty proxies")
        print("\r", end="", flush=True)
        self.proxies = [_ for _ in self.proxies if not any(_["server"].startswith(ip) for ip in self.blacklist)]
        self.test_latency(self.exclude_country, max_ping=max_ping)

    def start(self, quiet: bool = False, proxy: dict = None):
        if not self.proxies and not proxy:
            raise ValueError("empty proxies")
        if not proxy:
            if len(self.fast_proxy) > 1:
                proxies = self.fast_proxy
            else:
                proxies = self.proxies
            proxy = random.SystemRandom().choice(proxies)
            proxy = random.SystemRandom().choice(proxies)
        self.prev_proxy = proxy
        id = "{}_{}".format(proxy["server"], proxy["port"])
        config_fp = os.path.join("config", "{}_{}".format(self.clash_port, id), "config_ss.yaml")
        os.makedirs(os.path.dirname(config_fp), exist_ok=True)
        data = ["cache.db", "Country.mmdb"]
        for _ in data:
            shutil.copyfile(os.path.join(pkg_data_dir, _), os.path.join(os.path.dirname(config_fp), _))
        _proxy = proxy.copy()
        _proxy["name"] = "PROXY"
        open(config_fp, "wb").write(('''\
port: {}
allow-lan: true
mode: rule
proxies:
  - {}
rules:
  - MATCH,PROXY
'''.format(
            self.clash_port,
            "\n".join("{}{}".format("    " if i else "", _) for i, _ in enumerate(yaml.dump(_proxy).splitlines())),
        )).encode())
        if not quiet:
            self.clash = Popen([self.clash_bin, "-f", config_fp, "-d", os.path.dirname(config_fp)], close_fds=not IS_WIN32)
        else:
            self.clash = Popen([self.clash_bin, "-f", config_fp, "-d", os.path.dirname(config_fp)], stdout=PIPE, stderr=PIPE, close_fds=not IS_WIN32)
        pid = self.clash.pid
        if self.multiple:
            print("\rstarted clash", self.clash_port, id, "pid", pid, end="")
        else:
            print("started clash", self.clash_port, id, "pid", pid)
        while not self.terminate:
            time.sleep(1)
        self.clash.terminate()
        time.sleep(1)
        shutil.rmtree(os.path.dirname(config_fp))
        self.terminate = False
        self.clash = None

    def start_fast_proxy(self, *args, **kwargs):
        return self.start(*args, **kwargs)

    def start_gse_proxy(self, quiet: bool = False):
        if not self.gse_proxy:
            raise ValueError("empty proxies")
        proxies = self.gse_proxy
        proxy = random.SystemRandom().choice(proxies)
        proxy = random.SystemRandom().choice(proxies)
        return self.start(quiet=quiet, proxy=proxy)

    def stop(self):
        if self.clash:
            if self.multiple:
                print("\rclash terminating", self.clash.pid, end="")
            else:
                print("clash terminating", self.clash.pid)
            self.terminate = True
            while self.terminate:
                time.sleep(1)
            if self.multiple:
                print("\rclash terminated", end="")
            else:
                print("clash terminated")

    def test_latency(self, exclude_country, max_ping):
        print("clash testing latency, please wait")
        tw = threadwrapper.ThreadWrapper(threading.Semaphore(2**4))
        self.proxies = [_ for _ in self.proxies if "country" in _ and _["country"][-2:] not in exclude_country]
        ping_cache = {}
        if os.path.isfile("ping_cache.json") and os.path.getmtime("ping_cache.json")+60*60*24 > time.time():
            ping_cache = json.loads(open("ping_cache.json", "rb").read().decode())
        for i, proxy in enumerate(self.proxies):
            def job(i, proxy):
                try:
                    if i%100==0:
                        print("\r", i+1, len(self.proxies), "pinging", proxy["server"], end="", flush=True)
                    if proxy["server"] in ping_cache:
                        proxy["ping"] = ping_cache[proxy["server"]]
                    else:
                        pings = [999]
                        for i in range(0, 5):
                            p = ping(proxy["server"])
                            pings.append(p)
                            if p < 999:
                                break
                        proxy["ping"] = min(pings)
                        ping_cache[proxy["server"]] = proxy["ping"]
                    # print("\r", i+1, len(self.proxies), "pinging", proxy["server"], proxy["ping"], end="", flush=True)
                except:
                    proxy["ping"] = 999
            tw.add(job=def_template(job, i, proxy))
        tw.wait()
        open("ping_cache.json", "wb").write(json.dumps(ping_cache).encode())
        def fsp(_):
            return _["server"]
            return "{}:{}".format(_["server"], _["port"])
        exp = []
        self.proxies = [_ for _ in self.proxies if _["ping"] <= max_ping]
        self.proxies = [[_,exp.append(fsp(_))][0] for _ in self.proxies if _["ping"] <= max_ping and fsp(_) not in exp]
        self.proxies = sorted(self.proxies, key=lambda x: x["ping"])
        # print(len(self.proxies))
